/*******************************************************************************
* File Name: RI.c  
* Version 1.60
*
*  Description:
*   API for controlling the state of an interrupt.
*
*
*  Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/


#include <CYDEVICE.H>
#include <CYDEVICE_TRM.H>
#include <CYLIB.H>
#include <RI.H>


/*******************************************************************************
*  Place your includes, defines and code here 
********************************************************************************/
/* `#START RI_intc` */

//#include <device.h>
#include "UART_1.h"
#include "sfrs.h"


/* `#END` */

#ifndef CYINT_IRQ_BASE
#define CYINT_IRQ_BASE	16
#endif
#ifndef CYINT_VECT_TABLE
#define CYINT_VECT_TABLE    ((cyisraddress **) CYREG_NVIC_VECT_OFFSET)
#endif

/* Declared in startup, used to set unused interrupts to. */
CY_ISR_PROTO(IntDefaultHandler);

/*******************************************************************************
* Function Name: RI_Start
********************************************************************************
* Summary:
*  Set up the interrupt and enable it.
*
* Parameters:  
*   void.
*
*
* Return:
*   void.
*
*******************************************************************************/
void RI_Start(void)
{
    /* For all we know the interrupt is active. */
    RI_Disable();

    /* Set the ISR to point to the RI Interrupt. */
    RI_SetVector(RI_Interrupt);

    /* Set the priority. */
    RI_SetPriority(RI_INTC_PRIOR_NUMBER);

    /* Enable it. */
    RI_Enable();
}

/*******************************************************************************
* Function Name: RI_StartEx
********************************************************************************
* Summary:
*  Set up the interrupt and enable it.
*
* Parameters:  
*   address: Address of the ISR to set in the interrupt vector table.
*
*
* Return:
*   void.
*
*******************************************************************************/
void RI_StartEx(cyisraddress address)
{
    /* For all we know the interrupt is active. */
    RI_Disable();

    /* Set the ISR to point to the RI Interrupt. */
    RI_SetVector(address);

    /* Set the priority. */
    RI_SetPriority(RI_INTC_PRIOR_NUMBER);

    /* Enable it. */
    RI_Enable();
}

/*******************************************************************************
* Function Name: RI_Stop
********************************************************************************
* Summary:
*   Disables and removes the interrupt.
*
* Parameters:  
*
*
* Return:
*   void.
*
*******************************************************************************/
void RI_Stop(void) 
{
    /* Disable this interrupt. */
    RI_Disable();

    /* Set the ISR to point to the passive one. */
    RI_SetVector(IntDefaultHandler);
}

/*******************************************************************************
* Function Name: RI_Interrupt
********************************************************************************
* Summary:
*   The default Interrupt Service Routine for RI.
*
*   Add custom code between the coments to keep the next version of this file
*   from over writting your code.
*
*
*
* Parameters:  
*
*
* Return:
*   void.
*
*******************************************************************************/
CY_ISR(RI_Interrupt)
{
    /*  Place your Interrupt code here. */
    /* `#START RI_Interrupt` */
	
	extern uint8 InternalRAM[0xFF];

	InternalRAM[SCON] |= 0b00000001;	// set RI bit
	InternalRAM[SBUF_RX] = UART_1_GetChar();

    /* `#END` */
}

/*******************************************************************************
* Function Name: RI_SetVector
********************************************************************************
* Summary:
*   Change the ISR vector for the Interrupt. Note calling RI_Start
*   will override any effect this method would have had. To set the vector before
*   the component has been started use RI_StartEx instead.
*
*
* Parameters:
*   address: Address of the ISR to set in the interrupt vector table.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void RI_SetVector(cyisraddress address) 
{
    cyisraddress * ramVectorTable;

    ramVectorTable = (cyisraddress *) *CYINT_VECT_TABLE;

    ramVectorTable[CYINT_IRQ_BASE + RI__INTC_NUMBER] = address;
}

/*******************************************************************************
* Function Name: RI_GetVector
********************************************************************************
* Summary:
*   Gets the "address" of the current ISR vector for the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   Address of the ISR in the interrupt vector table.
*
*
*******************************************************************************/
cyisraddress RI_GetVector(void) 
{
    cyisraddress * ramVectorTable;

    ramVectorTable = (cyisraddress *) *CYINT_VECT_TABLE;

    return ramVectorTable[CYINT_IRQ_BASE + RI__INTC_NUMBER];
}

/*******************************************************************************
* Function Name: RI_SetPriority
********************************************************************************
* Summary:
*   Sets the Priority of the Interrupt. Note calling RI_Start
*   or RI_StartEx will override any effect this method would have had. 
*	This method should only be called after RI_Start or 
*	RI_StartEx has been called. To set the initial
*	priority for the component use the cydwr file in the tool.
*
*
* Parameters:
*   priority: Priority of the interrupt. 0 - 7, 0 being the highest.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void RI_SetPriority(uint8 priority) 
{
    *RI_INTC_PRIOR = priority << 5;
}

/*******************************************************************************
* Function Name: RI_GetPriority
********************************************************************************
* Summary:
*   Gets the Priority of the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   Priority of the interrupt. 0 - 7, 0 being the highest.
*
*
*******************************************************************************/
uint8 RI_GetPriority(void) 
{
    uint8 priority;


    priority = *RI_INTC_PRIOR >> 5;

    return priority;
}

/*******************************************************************************
* Function Name: RI_Enable
********************************************************************************
* Summary:
*   Enables the interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void RI_Enable(void) 
{
    /* Enable the general interrupt. */
    *RI_INTC_SET_EN = RI__INTC_MASK;
}

/*******************************************************************************
* Function Name: RI_GetState
********************************************************************************
* Summary:
*   Gets the state (enabled, disabled) of the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   1 if enabled, 0 if disabled.
*
*
*******************************************************************************/
uint8 RI_GetState(void) 
{
    /* Get the state of the general interrupt. */
    return (*RI_INTC_SET_EN & RI__INTC_MASK) ? 1:0;
}

/*******************************************************************************
* Function Name: RI_Disable
********************************************************************************
* Summary:
*   Disables the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void RI_Disable(void) 
{
    /* Disable the general interrupt. */
    *RI_INTC_CLR_EN = RI__INTC_MASK;
}

/*******************************************************************************
* Function Name: RI_SetPending
********************************************************************************
* Summary:
*   Causes the Interrupt to enter the pending state, a software method of
*   generating the interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void RI_SetPending(void) 
{
    *RI_INTC_SET_PD = RI__INTC_MASK;
}

/*******************************************************************************
* Function Name: RI_ClearPending
********************************************************************************
* Summary:
*   Clears a pending interrupt.
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void RI_ClearPending(void) 
{
    *RI_INTC_CLR_PD = RI__INTC_MASK;
}
