/*******************************************************************************
* File Name: Reset_int.c  
* Version 1.60
*
*  Description:
*   API for controlling the state of an interrupt.
*
*
*  Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/


#include <CYDEVICE.H>
#include <CYDEVICE_TRM.H>
#include <CYLIB.H>
#include <Reset_int.H>


/*******************************************************************************
*  Place your includes, defines and code here 
********************************************************************************/
/* `#START Reset_int_intc` */

/* `#END` */

#ifndef CYINT_IRQ_BASE
#define CYINT_IRQ_BASE	16
#endif
#ifndef CYINT_VECT_TABLE
#define CYINT_VECT_TABLE    ((cyisraddress **) CYREG_NVIC_VECT_OFFSET)
#endif

/* Declared in startup, used to set unused interrupts to. */
CY_ISR_PROTO(IntDefaultHandler);

/*******************************************************************************
* Function Name: Reset_int_Start
********************************************************************************
* Summary:
*  Set up the interrupt and enable it.
*
* Parameters:  
*   void.
*
*
* Return:
*   void.
*
*******************************************************************************/
void Reset_int_Start(void)
{
    /* For all we know the interrupt is active. */
    Reset_int_Disable();

    /* Set the ISR to point to the Reset_int Interrupt. */
    Reset_int_SetVector(Reset_int_Interrupt);

    /* Set the priority. */
    Reset_int_SetPriority(Reset_int_INTC_PRIOR_NUMBER);

    /* Enable it. */
    Reset_int_Enable();
}

/*******************************************************************************
* Function Name: Reset_int_StartEx
********************************************************************************
* Summary:
*  Set up the interrupt and enable it.
*
* Parameters:  
*   address: Address of the ISR to set in the interrupt vector table.
*
*
* Return:
*   void.
*
*******************************************************************************/
void Reset_int_StartEx(cyisraddress address)
{
    /* For all we know the interrupt is active. */
    Reset_int_Disable();

    /* Set the ISR to point to the Reset_int Interrupt. */
    Reset_int_SetVector(address);

    /* Set the priority. */
    Reset_int_SetPriority(Reset_int_INTC_PRIOR_NUMBER);

    /* Enable it. */
    Reset_int_Enable();
}

/*******************************************************************************
* Function Name: Reset_int_Stop
********************************************************************************
* Summary:
*   Disables and removes the interrupt.
*
* Parameters:  
*
*
* Return:
*   void.
*
*******************************************************************************/
void Reset_int_Stop(void) 
{
    /* Disable this interrupt. */
    Reset_int_Disable();

    /* Set the ISR to point to the passive one. */
    Reset_int_SetVector(IntDefaultHandler);
}

/*******************************************************************************
* Function Name: Reset_int_Interrupt
********************************************************************************
* Summary:
*   The default Interrupt Service Routine for Reset_int.
*
*   Add custom code between the coments to keep the next version of this file
*   from over writting your code.
*
*
*
* Parameters:  
*
*
* Return:
*   void.
*
*******************************************************************************/
CY_ISR(Reset_int_Interrupt)
{
    /*  Place your Interrupt code here. */
    /* `#START Reset_int_Interrupt` */

	CySoftwareReset();

    /* `#END` */
}

/*******************************************************************************
* Function Name: Reset_int_SetVector
********************************************************************************
* Summary:
*   Change the ISR vector for the Interrupt. Note calling Reset_int_Start
*   will override any effect this method would have had. To set the vector before
*   the component has been started use Reset_int_StartEx instead.
*
*
* Parameters:
*   address: Address of the ISR to set in the interrupt vector table.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void Reset_int_SetVector(cyisraddress address) 
{
    cyisraddress * ramVectorTable;

    ramVectorTable = (cyisraddress *) *CYINT_VECT_TABLE;

    ramVectorTable[CYINT_IRQ_BASE + Reset_int__INTC_NUMBER] = address;
}

/*******************************************************************************
* Function Name: Reset_int_GetVector
********************************************************************************
* Summary:
*   Gets the "address" of the current ISR vector for the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   Address of the ISR in the interrupt vector table.
*
*
*******************************************************************************/
cyisraddress Reset_int_GetVector(void) 
{
    cyisraddress * ramVectorTable;

    ramVectorTable = (cyisraddress *) *CYINT_VECT_TABLE;

    return ramVectorTable[CYINT_IRQ_BASE + Reset_int__INTC_NUMBER];
}

/*******************************************************************************
* Function Name: Reset_int_SetPriority
********************************************************************************
* Summary:
*   Sets the Priority of the Interrupt. Note calling Reset_int_Start
*   or Reset_int_StartEx will override any effect this method would have had. 
*	This method should only be called after Reset_int_Start or 
*	Reset_int_StartEx has been called. To set the initial
*	priority for the component use the cydwr file in the tool.
*
*
* Parameters:
*   priority: Priority of the interrupt. 0 - 7, 0 being the highest.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void Reset_int_SetPriority(uint8 priority) 
{
    *Reset_int_INTC_PRIOR = priority << 5;
}

/*******************************************************************************
* Function Name: Reset_int_GetPriority
********************************************************************************
* Summary:
*   Gets the Priority of the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   Priority of the interrupt. 0 - 7, 0 being the highest.
*
*
*******************************************************************************/
uint8 Reset_int_GetPriority(void) 
{
    uint8 priority;


    priority = *Reset_int_INTC_PRIOR >> 5;

    return priority;
}

/*******************************************************************************
* Function Name: Reset_int_Enable
********************************************************************************
* Summary:
*   Enables the interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void Reset_int_Enable(void) 
{
    /* Enable the general interrupt. */
    *Reset_int_INTC_SET_EN = Reset_int__INTC_MASK;
}

/*******************************************************************************
* Function Name: Reset_int_GetState
********************************************************************************
* Summary:
*   Gets the state (enabled, disabled) of the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   1 if enabled, 0 if disabled.
*
*
*******************************************************************************/
uint8 Reset_int_GetState(void) 
{
    /* Get the state of the general interrupt. */
    return (*Reset_int_INTC_SET_EN & Reset_int__INTC_MASK) ? 1:0;
}

/*******************************************************************************
* Function Name: Reset_int_Disable
********************************************************************************
* Summary:
*   Disables the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void Reset_int_Disable(void) 
{
    /* Disable the general interrupt. */
    *Reset_int_INTC_CLR_EN = Reset_int__INTC_MASK;
}

/*******************************************************************************
* Function Name: Reset_int_SetPending
********************************************************************************
* Summary:
*   Causes the Interrupt to enter the pending state, a software method of
*   generating the interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void Reset_int_SetPending(void) 
{
    *Reset_int_INTC_SET_PD = Reset_int__INTC_MASK;
}

/*******************************************************************************
* Function Name: Reset_int_ClearPending
********************************************************************************
* Summary:
*   Clears a pending interrupt.
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void Reset_int_ClearPending(void) 
{
    *Reset_int_INTC_CLR_PD = Reset_int__INTC_MASK;
}
