/*******************************************************************************
* File Name: TI.c  
* Version 1.60
*
*  Description:
*   API for controlling the state of an interrupt.
*
*
*  Note:
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/


#include <CYDEVICE.H>
#include <CYDEVICE_TRM.H>
#include <CYLIB.H>
#include <TI.H>


/*******************************************************************************
*  Place your includes, defines and code here 
********************************************************************************/
/* `#START TI_intc` */

#include "UART_1.h"
#include "sfrs.h"

/* `#END` */

#ifndef CYINT_IRQ_BASE
#define CYINT_IRQ_BASE	16
#endif
#ifndef CYINT_VECT_TABLE
#define CYINT_VECT_TABLE    ((cyisraddress **) CYREG_NVIC_VECT_OFFSET)
#endif

/* Declared in startup, used to set unused interrupts to. */
CY_ISR_PROTO(IntDefaultHandler);

/*******************************************************************************
* Function Name: TI_Start
********************************************************************************
* Summary:
*  Set up the interrupt and enable it.
*
* Parameters:  
*   void.
*
*
* Return:
*   void.
*
*******************************************************************************/
void TI_Start(void)
{
    /* For all we know the interrupt is active. */
    TI_Disable();

    /* Set the ISR to point to the TI Interrupt. */
    TI_SetVector(TI_Interrupt);

    /* Set the priority. */
    TI_SetPriority(TI_INTC_PRIOR_NUMBER);

    /* Enable it. */
    TI_Enable();
}

/*******************************************************************************
* Function Name: TI_StartEx
********************************************************************************
* Summary:
*  Set up the interrupt and enable it.
*
* Parameters:  
*   address: Address of the ISR to set in the interrupt vector table.
*
*
* Return:
*   void.
*
*******************************************************************************/
void TI_StartEx(cyisraddress address)
{
    /* For all we know the interrupt is active. */
    TI_Disable();

    /* Set the ISR to point to the TI Interrupt. */
    TI_SetVector(address);

    /* Set the priority. */
    TI_SetPriority(TI_INTC_PRIOR_NUMBER);

    /* Enable it. */
    TI_Enable();
}

/*******************************************************************************
* Function Name: TI_Stop
********************************************************************************
* Summary:
*   Disables and removes the interrupt.
*
* Parameters:  
*
*
* Return:
*   void.
*
*******************************************************************************/
void TI_Stop(void) 
{
    /* Disable this interrupt. */
    TI_Disable();

    /* Set the ISR to point to the passive one. */
    TI_SetVector(IntDefaultHandler);
}

/*******************************************************************************
* Function Name: TI_Interrupt
********************************************************************************
* Summary:
*   The default Interrupt Service Routine for TI.
*
*   Add custom code between the coments to keep the next version of this file
*   from over writting your code.
*
*
*
* Parameters:  
*
*
* Return:
*   void.
*
*******************************************************************************/
CY_ISR(TI_Interrupt)
{
    /*  Place your Interrupt code here. */
    /* `#START TI_Interrupt` */

	extern uint8 InternalRAM[0xFF];
	
	InternalRAM[SCON] |= 0b00000010;	//set TI bit

    /* `#END` */
}

/*******************************************************************************
* Function Name: TI_SetVector
********************************************************************************
* Summary:
*   Change the ISR vector for the Interrupt. Note calling TI_Start
*   will override any effect this method would have had. To set the vector before
*   the component has been started use TI_StartEx instead.
*
*
* Parameters:
*   address: Address of the ISR to set in the interrupt vector table.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void TI_SetVector(cyisraddress address) 
{
    cyisraddress * ramVectorTable;

    ramVectorTable = (cyisraddress *) *CYINT_VECT_TABLE;

    ramVectorTable[CYINT_IRQ_BASE + TI__INTC_NUMBER] = address;
}

/*******************************************************************************
* Function Name: TI_GetVector
********************************************************************************
* Summary:
*   Gets the "address" of the current ISR vector for the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   Address of the ISR in the interrupt vector table.
*
*
*******************************************************************************/
cyisraddress TI_GetVector(void) 
{
    cyisraddress * ramVectorTable;

    ramVectorTable = (cyisraddress *) *CYINT_VECT_TABLE;

    return ramVectorTable[CYINT_IRQ_BASE + TI__INTC_NUMBER];
}

/*******************************************************************************
* Function Name: TI_SetPriority
********************************************************************************
* Summary:
*   Sets the Priority of the Interrupt. Note calling TI_Start
*   or TI_StartEx will override any effect this method would have had. 
*	This method should only be called after TI_Start or 
*	TI_StartEx has been called. To set the initial
*	priority for the component use the cydwr file in the tool.
*
*
* Parameters:
*   priority: Priority of the interrupt. 0 - 7, 0 being the highest.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void TI_SetPriority(uint8 priority) 
{
    *TI_INTC_PRIOR = priority << 5;
}

/*******************************************************************************
* Function Name: TI_GetPriority
********************************************************************************
* Summary:
*   Gets the Priority of the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   Priority of the interrupt. 0 - 7, 0 being the highest.
*
*
*******************************************************************************/
uint8 TI_GetPriority(void) 
{
    uint8 priority;


    priority = *TI_INTC_PRIOR >> 5;

    return priority;
}

/*******************************************************************************
* Function Name: TI_Enable
********************************************************************************
* Summary:
*   Enables the interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void TI_Enable(void) 
{
    /* Enable the general interrupt. */
    *TI_INTC_SET_EN = TI__INTC_MASK;
}

/*******************************************************************************
* Function Name: TI_GetState
********************************************************************************
* Summary:
*   Gets the state (enabled, disabled) of the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   1 if enabled, 0 if disabled.
*
*
*******************************************************************************/
uint8 TI_GetState(void) 
{
    /* Get the state of the general interrupt. */
    return (*TI_INTC_SET_EN & TI__INTC_MASK) ? 1:0;
}

/*******************************************************************************
* Function Name: TI_Disable
********************************************************************************
* Summary:
*   Disables the Interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void TI_Disable(void) 
{
    /* Disable the general interrupt. */
    *TI_INTC_CLR_EN = TI__INTC_MASK;
}

/*******************************************************************************
* Function Name: TI_SetPending
********************************************************************************
* Summary:
*   Causes the Interrupt to enter the pending state, a software method of
*   generating the interrupt.
*
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void TI_SetPending(void) 
{
    *TI_INTC_SET_PD = TI__INTC_MASK;
}

/*******************************************************************************
* Function Name: TI_ClearPending
********************************************************************************
* Summary:
*   Clears a pending interrupt.
*
* Parameters:
*   void.
*
*
* Return:
*   void.
*
*
*******************************************************************************/
void TI_ClearPending(void) 
{
    *TI_INTC_CLR_PD = TI__INTC_MASK;
}
