/*******************************************************************************
* File Name: TI.h
* Version 1.60
*
*  Description:
*   Provides the function definitions for the Interrupt Controller.
*
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/
#if !defined(__TI_INTC_H__)
#define __TI_INTC_H__


#include <cytypes.h>
#include <cyfitter.h>

/* Interrupt Controller API. */
void TI_Start(void);
void TI_StartEx(cyisraddress address);
void TI_Stop(void) ;

CY_ISR_PROTO(TI_Interrupt);

void TI_SetVector(cyisraddress address) ;
cyisraddress TI_GetVector(void) ;

void TI_SetPriority(uint8 priority) ;
uint8 TI_GetPriority(void) ;

void TI_Enable(void) ;
uint8 TI_GetState(void) ;
void TI_Disable(void) ;

void TI_SetPending(void) ;
void TI_ClearPending(void) ;


/* Interrupt Controller Constants */

/* Address of the INTC.VECT[x] register that contains the Address of the TI ISR. */
#define TI_INTC_VECTOR            ((reg32 *) TI__INTC_VECT)

/* Address of the TI ISR priority. */
#define TI_INTC_PRIOR             ((reg8 *) TI__INTC_PRIOR_REG)

/* Priority of the TI interrupt. */
#define TI_INTC_PRIOR_NUMBER      TI__INTC_PRIOR_NUM

/* Address of the INTC.SET_EN[x] byte to bit enable TI interrupt. */
#define TI_INTC_SET_EN            ((reg32 *) TI__INTC_SET_EN_REG)

/* Address of the INTC.CLR_EN[x] register to bit clear the TI interrupt. */
#define TI_INTC_CLR_EN            ((reg32 *) TI__INTC_CLR_EN_REG)

/* Address of the INTC.SET_PD[x] register to set the TI interrupt state to pending. */
#define TI_INTC_SET_PD            ((reg32 *) TI__INTC_SET_PD_REG)

/* Address of the INTC.CLR_PD[x] register to clear the TI interrupt. */
#define TI_INTC_CLR_PD            ((reg32 *) TI__INTC_CLR_PD_REG)



/* __TI_INTC_H__ */
#endif
