/* ==============================================
 * Implementing bit-addressable bytes
 * ==============================================
*/

#if !defined(BIT_ADDRESSABLE_H)
#define BIT_ADDRESSABLE_H

#include <device.h>
#include "sfrs.h"
#include "bit_addressable.h"

#define bit_addressable_table_max	0xF7
#define bit_addressable(bit_address)	(bit_addressable_table[bit_address])

extern uint8 InternalRAM[0xFF];
extern uint8 bit_addressable_table[bit_addressable_table_max+1];

static inline uint8 get_bit_addressable(uint8 bit_address)
{
	uint8 byte_address;		// the byte with the bit we are addressing
	
	/* Bit-addressable region 0x20 --> 0x2F */
	switch (bit_address) {
		case 0x00:
		case 0x01:
		case 0x02:
		case 0x03:
		case 0x04:
		case 0x05:
		case 0x06:
		case 0x07:
			byte_address = 0x20; 
			break;
		case 0x08:
		case 0x09:
		case 0x0A:
		case 0x0B:
		case 0x0C:
		case 0x0D:
		case 0x0E:
		case 0x0F:
			byte_address = 0x21;
			break;
		case 0x10:
		case 0x11:
		case 0x12:
		case 0x13:
		case 0x14:
		case 0x15:
		case 0x16:
		case 0x17:
			byte_address = 0x22; 
			break;
		case 0x18:
		case 0x19:
		case 0x1A:
		case 0x1B:
		case 0x1C:
		case 0x1D:
		case 0x1E:
		case 0x1F:
			byte_address = 0x23;
			break;	
		case 0x20:
		case 0x21:
		case 0x22:
		case 0x23:
		case 0x24:
		case 0x25:
		case 0x26:
		case 0x27:
			byte_address = 0x24; 
			break;
		case 0x28:
		case 0x29:
		case 0x2A:
		case 0x2B:
		case 0x2C:
		case 0x2D:
		case 0x2E:
		case 0x2F:
			byte_address = 0x25;
			break;
		case 0x30:
		case 0x31:
		case 0x32:
		case 0x33:
		case 0x34:
		case 0x35:
		case 0x36:
		case 0x37:
			byte_address = 0x26; 
			break;
		case 0x38:
		case 0x39:
		case 0x3A:
		case 0x3B:
		case 0x3C:
		case 0x3D:
		case 0x3E:
		case 0x3F:
			byte_address = 0x27;
			break;
		case 0x40:
		case 0x41:
		case 0x42:
		case 0x43:
		case 0x44:
		case 0x45:
		case 0x46:
		case 0x47:
			byte_address = 0x28; 
			break;
		case 0x48:
		case 0x49:
		case 0x4A:
		case 0x4B:
		case 0x4C:
		case 0x4D:
		case 0x4E:
		case 0x4F:
			byte_address = 0x29;
			break;
		case 0x50:
		case 0x51:
		case 0x52:
		case 0x53:
		case 0x54:
		case 0x55:
		case 0x56:
		case 0x57:
			byte_address = 0x2A; 
			break;
		case 0x58:
		case 0x59:
		case 0x5A:
		case 0x5B:
		case 0x5C:
		case 0x5D:
		case 0x5E:
		case 0x5F:
			byte_address = 0x2B;
			break;
		case 0x60:
		case 0x61:
		case 0x62:
		case 0x63:
		case 0x64:
		case 0x65:
		case 0x66:
		case 0x67:
			byte_address = 0x2C; 
			break;
		case 0x68:
		case 0x69:
		case 0x6A:
		case 0x6B:
		case 0x6C:
		case 0x6D:
		case 0x6E:
		case 0x6F:
			byte_address = 0x2D;
			break;
		case 0x70:
		case 0x71:
		case 0x72:
		case 0x73:
		case 0x74:
		case 0x75:
		case 0x76:
		case 0x77:
			byte_address = 0x2E; 
			break;
		case 0x78:
		case 0x79:
		case 0x7A:
		case 0x7B:
		case 0x7C:
		case 0x7D:
		case 0x7E:
		case 0x7F:
			byte_address = 0x2F;
			break;
		
	/* Bit-addressable SFRs 0x80 --> 0xF0 */
	
		case 0x80:
		case 0x81:
		case 0x82:
		case 0x83:
		case 0x84:
		case 0x85:
		case 0x86:
		case 0x87:
			byte_address = P0; 
			break;
		case 0x88:
		case 0x89:
		case 0x8A:
		case 0x8B:
		case 0x8C:
		case 0x8D:
		case 0x8E:
		case 0x8F:
			byte_address = TCON;
			break;
		case 0x90:
		case 0x91:
		case 0x92:
		case 0x93:
		case 0x94:
		case 0x95:
		case 0x96:
		case 0x97:
			byte_address = P1; 
			break;
		case 0x98:
		case 0x99:
		case 0x9A:
		case 0x9B:
		case 0x9C:
		case 0x9D:
		case 0x9E:
		case 0x9F:
			byte_address = SCON;
			break;
		case 0xA0:
		case 0xA1:
		case 0xA2:
		case 0xA3:
		case 0xA4:
		case 0xA5:
		case 0xA6:
		case 0xA7:
			byte_address = P2; 
			break;
		case 0xA8:
		case 0xA9:
		case 0xAA:
		case 0xAB:
		case 0xAC:
		case 0xAD:
		case 0xAE:
		case 0xAF:
			byte_address = IE;
			break;
		case 0xB0:
		case 0xB1:
		case 0xB2:
		case 0xB3:
		case 0xB4:
		case 0xB5:
		case 0xB6:
		case 0xB7:
			byte_address = P3; 
			break;
		case 0xB8:
		case 0xB9:
		case 0xBA:
		case 0xBB:
		case 0xBC:
		case 0xBD:
		case 0xBE:
		case 0xBF:
			byte_address = IP;
			break;
		case 0xC0:
		case 0xC1:
		case 0xC2:
		case 0xC3:
		case 0xC4:
		case 0xC5:
		case 0xC6:
		case 0xC7:
			byte_address = ENP; 
			break;	
		case 0xD0:
		case 0xD1:
		case 0xD2:
		case 0xD3:
		case 0xD4:
		case 0xD5:
		case 0xD6:
		case 0xD7:
			byte_address = PSW; 
			break;
		case 0xE0:
		case 0xE1:
		case 0xE2:
		case 0xE3:
		case 0xE4:
		case 0xE5:
		case 0xE6:
		case 0xE7:
			byte_address = ACC; 
			break;
		case 0xF0:
		case 0xF1:
		case 0xF2:
		case 0xF3:
		case 0xF4:
		case 0xF5:
		case 0xF6:
		case 0xF7:
			byte_address = B; 
			break;
	}

	return byte_address;
}

static inline uint8 bit_check(uint8 bit_address)		// bit address is the address of the bit we want
{	
	uint8 current_byte;		// the byte with the bit we are addressing
	uint8 byte_address;		// the address of the byte with the bit we are addressing
	uint8 bit_index;		// the index (0-7) of the bit we are addressing
	uint8 bit_byte;		// the number 0x00 or 0x01 stored in that bit
	
	bit_index = (bit_address & 0b00000111);

	byte_address = bit_addressable(bit_address);
	current_byte = InternalRAM[byte_address];
	bit_byte = (current_byte >> bit_index) & 0b00000001;
	return bit_byte;
}

static inline void PSW_update_subb(uint8 x, uint8 y)
{
	int8 a;
	int8 b;
	
	if (x < y)
		InternalRAM[PSW] = (InternalRAM[PSW] | 0b10000000);
	else
		InternalRAM[PSW] = (InternalRAM[PSW] & 0b01111111);
	if ((x & 0b00001111) < (y & 0b00001111))
		InternalRAM[PSW] = (InternalRAM[PSW] | 0b01000000);
	else
		InternalRAM[PSW] = (InternalRAM[PSW] & 0b10111111);
	a = (int8)x;
	b = (int8)y;
	if (a < b)
		InternalRAM[PSW] = (InternalRAM[PSW] | 0b00000100);
	else
		InternalRAM[PSW] = (InternalRAM[PSW] & 0b11111011);
}

static inline void PSW_update_add(uint8 x, uint8 y)
{
	int8 a;
	int8 b;
	
	if (x > y)
		InternalRAM[PSW] = (InternalRAM[PSW] | 0b10000000);
	else
		InternalRAM[PSW] = (InternalRAM[PSW] & 0b01111111);
	if ((x & 0b00001111) > (y & 0b00001111))
		InternalRAM[PSW] = (InternalRAM[PSW] | 0b01000000);
	else
		InternalRAM[PSW] = (InternalRAM[PSW] & 0b10111111);
	a = (int8)x;
	b = (int8)y;
	if (a > b)
		InternalRAM[PSW] = (InternalRAM[PSW] | 0b00000100);
	else
		InternalRAM[PSW] = (InternalRAM[PSW] & 0b11111011);
}

static inline uint8 parity(uint8 acc)
{
	uint8 par;
	
	par = acc ^ (acc >> 1);
	par ^= (par >> 2);
	par ^= (par >> 4);
	par &= 0x01;
	
	return par;
}

static inline void init_bit_addressable()
{
	int ix ;
	for (ix = 0 ; ix <= bit_addressable_table_max ; ix++ )
		bit_addressable_table[ix] = get_bit_addressable(ix);
}

static inline void bit_toggle(uint8 bit_address)
{
	uint8 byte_address;		// the address of the byte with the bit we are addressing
	uint8 bit_index;		// the index (0-7) of the bit we are addressing
	
	bit_index = (bit_address & 0b00000111);
	
	byte_address = bit_addressable(bit_address);
	InternalRAM[byte_address] = (InternalRAM[byte_address] ^ (0b00000001 << bit_index));
}

static inline void bit_change(uint8 bit_address, uint8 set_or_clr)
{
	uint8 byte_address;		// the address of the byte with the bit we are addressing
	uint8 bit_index;		// the index (0-7) of the bit we are addressing
	
	bit_index = (bit_address & 0b00000111);
	
	byte_address = bit_addressable(bit_address);
	InternalRAM[byte_address] = (InternalRAM[byte_address] & ~(0b00000001 << bit_index)) | (set_or_clr << bit_index);
}


#endif
