/* ========================================
 *
 * Copyright YOUR COMPANY, THE YEAR
 * All Rights Reserved
 * UNPUBLISHED, LICENSED SOFTWARE.
 *
 * CONFIDENTIAL AND PROPRIETARY INFORMATION
 * WHICH IS THE PROPERTY OF your company.
 *
 * ========================================
*/

#if !defined(test_h)
#define test_h

#include <device.h>
#include <stdio.h>
#include <stdlib.h>
#include "stack.h"
#include "opcodes.h"
#include "comb_decomp.h"
#include "write_read_sfr.h"
#include "bit_addressable.h"
#include "sfrs.h"
#include "next_RAM.h"

#define Indirect_max	0x80
#define PROGRAM_SIZE	(32*1024)
#define NEXT_BYTE()	Program8051[pc++]

void ProgramLoop(void) {

	extern int syncPc;
	extern volatile uint16 globalPc;
	extern uint8 InternalRAM[0xFF];			/* Arrays store values of each location/register */
	extern uint8 ExternalRAM[0x2000];
	extern uint8 InternalRAM_indirect[0x80];  // Implement this space for @R0,@R1 instructions
	extern uint8 bit_addressable_table[bit_addressable_table_max + 1];
	extern uint8 Program8051[PROGRAM_SIZE];
	extern int EA_flag;

	uint8 x;
	uint16 z;
	uint8 pcl = 0x00;
	uint8 pch = 0x00;
	uint8 current_opcode = 0x00;
	uint8 current_index = 0x00;
	uint8 current_byte = 0x00;
	uint8 bit_address = 0x00;
	uint8 bit_value = 0x00;
	int8 offset;
	static uint16 dptr = 0x00;
	static uint16 pc = 0x00;
	int flag = 1;
	
	

     while(flag)
	{

/* ========================================
 * Switch through all possible opcodes
 * Next byte stored in current_opcode
 * Next address stored in current_index
 * Next byte to operate on stores in current_byte
 * ========================================
*/	
		CyGlobalIntDisable;
		if (syncPc) {
			pc = globalPc;		// update pc after interrupt
			syncPc = 0;
		}
		
		current_opcode = NEXT_BYTE();
	
		switch(current_opcode)
		{
			case NOP: // how do we generate a NOP? Need to know timing. Test this so its 1us.
				break;
			case AJMP_01:
			case AJMP_21:
			case AJMP_41:
			case AJMP_61:
			case AJMP_81:
			case AJMP_A1:
			case AJMP_C1:
			case AJMP_E1:
				current_byte = NEXT_BYTE();
				pch = decomp_high(pc + 2); 		/* The address of the next opcode */
				pcl = current_byte;
				pch = ((current_opcode & 0b11100000) >> 5) | (pch & 0b11111000);
				pc = comb(pch,pcl);
				break;
			case LJMP:
				current_byte = NEXT_BYTE();
				pch = current_byte;
				current_byte = NEXT_BYTE();
				pcl = current_byte;
				pc = comb(pch,pcl);
				break;
			case RR:
				InternalRAM[ACC] = (InternalRAM[ACC] << 7) | (InternalRAM[ACC] >> 1);
				break;
			case INC_A:
				++InternalRAM[ACC];
				break;
			case INC_dr:
				current_index = NEXT_BYTE();
				++InternalRAM[current_index];
				write_sfr_byte(current_index);
				break;
			case INC_atR0:
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					++InternalRAM[InternalRAM[8*current_Rbank]];
				else
					++InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max];
				break;
			case INC_atR1:
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					++InternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
				else
					++InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max];
				break;
			case INC_R0:
			case INC_R1:
			case INC_R2:
			case INC_R3:
			case INC_R4:
			case INC_R5:
			case INC_R6:
			case INC_R7:
				++InternalRAM[(current_opcode - 0x08) + 8*current_Rbank];
				break;
			case JBC:
				bit_address = NEXT_BYTE();
				bit_value = bit_check(bit_address);
				offset = NEXT_BYTE();
				if (bit_value == 0x01) {
					bit_change(bit_address, bit_value);
					pc += offset;
				}
				break;
			case ACALL_11:
			case ACALL_31:
			case ACALL_51:
			case ACALL_71:
			case ACALL_91:
			case ACALL_B1:
			case ACALL_D1:
			case ACALL_F1:
				//if (!EA_flag)
				//	pc = globalPc;
				pcl = decomp_low(pc + 2);
				push(pcl);
				pch = decomp_high(pc + 2);
				push(pch);
				pcl = NEXT_BYTE();
				pch = ((current_opcode & 0b11100000) >> 5) | (pch & 0b11111000);
				pc = comb(pch,pcl);
				break;
			case LCALL:
				//if (!EA_flag)
				//	pc = globalPc;
				pcl = decomp_low(pc + 2);
				push(pcl);
				pch = decomp_high(pc + 2);
				push(pch);
				pch = NEXT_BYTE();
				pcl = NEXT_BYTE();
				pc = comb(pch,pcl);
				break;
			case RRC:
				InternalRAM[PSW] = (InternalRAM[PSW] & 0b01111111) | (InternalRAM[ACC] << 7); 		/* rotate bit 0 of ACC into CY */
				InternalRAM[ACC] = (InternalRAM[ACC] >> 1) | cy;								/* right rotate, carry into bit 7 */
				break;
			case DEC_A:
				--InternalRAM[ACC];
				break;
			case DEC_dr:
				current_index = NEXT_BYTE();
				--InternalRAM[current_index];
				write_sfr_byte(current_index);
				break;
			case DEC_atR0:
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					--InternalRAM[InternalRAM[8*current_Rbank]];
				else
					--InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max];
				break;
			case DEC_atR1:
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					--InternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
				else
					--InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max];
				break;
			case DEC_R0:
			case DEC_R1:
			case DEC_R2:
			case DEC_R3:
			case DEC_R4:
			case DEC_R5:
			case DEC_R6:
			case DEC_R7:
				--InternalRAM[(current_opcode - 0x18) + 8*current_Rbank];
				break;
			case JB:
				bit_address = NEXT_BYTE();
				bit_value = bit_check(bit_address);
				offset = NEXT_BYTE();
				if (bit_value == 0x01) {
					pc += offset;
				}
				break;
			case RET:
				pch = pop();
				pcl = pop();
				pc = comb(pch,pcl);
				break;
			case RL:
				InternalRAM[ACC] = (InternalRAM[ACC] >> 7) | (InternalRAM[ACC] << 1);
				break;
			case ADD_im:
				current_byte = NEXT_BYTE();
				x = InternalRAM[ACC];
				InternalRAM[ACC] += current_byte;
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case ADD_dr:
				current_index = NEXT_BYTE();
				InternalRAM[current_index] = read_sfr(current_index);
				x = InternalRAM[ACC];
				InternalRAM[ACC] += InternalRAM[current_index];
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case ADD_atR0:
				x = InternalRAM[ACC];
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] += InternalRAM[InternalRAM[8*current_Rbank]];
				else
					InternalRAM[ACC] += InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max];
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case ADD_atR1:
				x = InternalRAM[ACC];
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] += InternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
				else
					InternalRAM[ACC] += InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max];
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case ADD_R0:
			case ADD_R1:
			case ADD_R2:
			case ADD_R3:
			case ADD_R4:
			case ADD_R5:
			case ADD_R6:
			case ADD_R7:
				x = InternalRAM[ACC];
				InternalRAM[ACC] += InternalRAM[(current_opcode - 0x28) + 8*current_Rbank];
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case JNB:
				bit_address = NEXT_BYTE();
				bit_value = bit_check(bit_address);
				offset = NEXT_BYTE();
				if (bit_value == 0x00) {
					pc += offset;
				}
				break;
			case RETI:
				pch = pop();
				pcl = pop();
				pc = comb(pch,pcl);
				flag = 0;
				// restore interrupt logic to accept other interrupts ??
				break;
			case RLC:
				InternalRAM[PSW] = (InternalRAM[PSW] & 0b01111111) | (InternalRAM[ACC] & 0b10000000); 		// rotate bit 7 of ACC into CY
				InternalRAM[ACC] = (InternalRAM[ACC] << 1) | (cy >> 7);								// left rotate, carry into bit 0 
				break;
			case ADDC_im:
				x = InternalRAM[ACC];
				current_byte = NEXT_BYTE();
				InternalRAM[ACC] += current_byte + cy;
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case ADDC_dr:
				x = InternalRAM[ACC];
				current_index = NEXT_BYTE();
				InternalRAM[current_index] = read_sfr(current_index);
				InternalRAM[ACC] += InternalRAM[current_index] + cy;
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case ADDC_atR0:
				x = InternalRAM[ACC];
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] += InternalRAM[InternalRAM[8*current_Rbank]] + cy;
				else
					InternalRAM[ACC] += InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] + cy;
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case ADDC_atR1:
				x = InternalRAM[ACC];
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] += InternalRAM[InternalRAM[0x01 + 8*current_Rbank]] + cy;
				else
					InternalRAM[ACC] += InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] + cy;
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case ADDC_R0:
			case ADDC_R1:
			case ADDC_R2:
			case ADDC_R3:
			case ADDC_R4:
			case ADDC_R5:
			case ADDC_R6:
			case ADDC_R7:
				x = InternalRAM[ACC];
				InternalRAM[ACC] = InternalRAM[ACC] + InternalRAM[(current_opcode - 0x38) + 8*current_Rbank] + cy;
				PSW_update_add(x,InternalRAM[ACC]);
				break;
			case JC:
				offset = NEXT_BYTE();
				if (cy == 1) {
					pc += offset;
				}
				break;
			case ORL_dr_A:		// read from output data latch
				current_index = NEXT_BYTE();
				InternalRAM[current_index] |= InternalRAM[ACC];
				write_sfr_byte(current_index);
				break;
			case ORL_dr_im:
				current_index = NEXT_BYTE();
				current_byte = NEXT_BYTE();
				InternalRAM[current_index] |= current_byte;
				write_sfr_byte(current_index);
				break;
			case ORL_im:
				current_byte = NEXT_BYTE();
				InternalRAM[ACC] |= current_byte;
				break;
			case ORL_dr:
				current_index = NEXT_BYTE();
				InternalRAM[ACC] |= InternalRAM[current_index];
				break;
			case ORL_atR0:
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] |= InternalRAM[InternalRAM[8*current_Rbank]];
				else
					InternalRAM[ACC] |= InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max];
				break;
			case ORL_atR1:
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] |= InternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
				else
					InternalRAM[ACC] |= InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max];
				break;
			case ORL_R0:
			case ORL_R1:
			case ORL_R2:
			case ORL_R3:
			case ORL_R4:
			case ORL_R5:
			case ORL_R6:
			case ORL_R7:
				InternalRAM[ACC] |= InternalRAM[(current_opcode - 0x48) + 8*current_Rbank];
				break;
			case JNC:
				offset = NEXT_BYTE();
				if (cy == 0) {
					pc += offset;
				}
				break;
			case ANL_dr_A:
				current_index = NEXT_BYTE();
				InternalRAM[current_index] &= InternalRAM[ACC];
				write_sfr_byte(current_index);
				break;
			case ANL_dr_im:
				current_index = NEXT_BYTE();
				current_byte = NEXT_BYTE();
				InternalRAM[current_index] &= current_byte;
				write_sfr_byte(current_index);
				break;
			case ANL_im:
				current_byte = NEXT_BYTE();
				InternalRAM[ACC] &= current_byte;
				break;
			case ANL_dr:
				current_index = NEXT_BYTE();
				InternalRAM[ACC] &= InternalRAM[current_index];
				break;
			case ANL_atR0:
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] &= InternalRAM[InternalRAM[8*current_Rbank]];
				else
					InternalRAM[ACC] &= InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max];
				break;
			case ANL_atR1:
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] &= InternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
				else
					InternalRAM[ACC] &= InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max];
				break;
			case ANL_R0:
			case ANL_R1:
			case ANL_R2:
			case ANL_R3:
			case ANL_R4:
			case ANL_R5:
			case ANL_R6:
			case ANL_R7:
				InternalRAM[ACC] &= InternalRAM[(current_opcode - 0x58) + 8*current_Rbank];
				break;
			case JZ:
				offset = NEXT_BYTE();
				if (InternalRAM[ACC] == 0x00) {
					pc += offset;
				}
				break;
			case XRL_dr_A:
				current_index = NEXT_BYTE();
				InternalRAM[current_index] = InternalRAM[current_index] ^ InternalRAM[ACC];
				write_sfr_byte(current_index);
				break;
			case XRL_dr_im:
				current_index = NEXT_BYTE();
				current_byte = NEXT_BYTE();
				InternalRAM[current_index] = InternalRAM[current_index] ^ current_byte;
				write_sfr_byte(current_index);
				break;
			case XRL_im:
				current_byte = NEXT_BYTE();
				InternalRAM[ACC] ^= current_byte;
				break;
			case XRL_dr:
				current_index = NEXT_BYTE();
				InternalRAM[ACC] ^= InternalRAM[current_index];
				break;
			case XRL_atR0:
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] ^= InternalRAM[InternalRAM[8*current_Rbank]];
				else 
					InternalRAM[ACC] ^= InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max];
				break;
			case XRL_atR1:
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] ^= InternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
				else 
					InternalRAM[ACC] ^= InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max];
				break;
			case XRL_R0:
			case XRL_R1:
			case XRL_R2:
			case XRL_R3:
			case XRL_R4:
			case XRL_R5:
			case XRL_R6:
			case XRL_R7:
				InternalRAM[ACC] ^= InternalRAM[(current_opcode - 0x68) + 8*current_Rbank];
				break;
			case JNZ:
				offset = NEXT_BYTE();
				if (InternalRAM[ACC] != 0x00) {
					pc += offset;
				}
				break;
			case ORL_bit:
				bit_address = NEXT_BYTE();
				bit_value = bit_check(bit_address);
				InternalRAM[PSW] = (InternalRAM[PSW] & 0b01111111) | ((bit_value | cy) << 7);
				write_sfr_byte(bit_addressable(bit_address));
				break;
			case JMP:
				dptr = comb(InternalRAM[DPH],InternalRAM[DPL]);
				pc = InternalRAM[ACC] + dptr;
				break;
			case MOV_A_im:
				current_byte = NEXT_BYTE();
				InternalRAM[ACC] = current_byte;
				break;
			case MOV_dr_im:
				current_index = NEXT_BYTE();
				current_byte = NEXT_BYTE();
				InternalRAM[current_index] = current_byte;
				write_sfr_byte(current_index);
				break;
			case MOV_atR0_im:
				current_byte = NEXT_BYTE();
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[InternalRAM[8*current_Rbank]] = current_byte;
				else
					InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] = current_byte;
				break;
			case MOV_atR1_im:
				current_byte = NEXT_BYTE();
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[InternalRAM[0x01 + 8*current_Rbank]] = current_byte;
				else 
					InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] = current_byte;
				break;
			case MOV_R0_im:
			case MOV_R1_im:
			case MOV_R2_im:
			case MOV_R3_im:
			case MOV_R4_im:
			case MOV_R5_im:
			case MOV_R6_im:
			case MOV_R7_im:
				current_byte = NEXT_BYTE();
				InternalRAM[(current_opcode - 0x78) + 8*current_Rbank] = current_byte;
				break;
				
			/* MIDPOINT */
			
			case SJMP:
				offset = NEXT_BYTE();
				pc += offset;	//changed
				break;	
			case ANL_bit:
				bit_address = NEXT_BYTE();
				bit_value = bit_check(bit_address);
				InternalRAM[PSW] = (InternalRAM[PSW] & 0b01111111) | ((bit_value & cy) << 7);
				break;
			case MOVC_PC:
				InternalRAM[ACC] = Program8051[pc + InternalRAM[ACC]];
				break;
			case DIV: 
				x = InternalRAM[ACC];
				InternalRAM[ACC] /= InternalRAM[B];
				InternalRAM[PSW] &= 0b01111011;		/* clear CY and OV */
				if (InternalRAM[B] == 0x00)
					InternalRAM[PSW] |= 0b00000100;		
				InternalRAM[B] = x % InternalRAM[B];
				break;
			case MOV_dr_dr:
				current_index = NEXT_BYTE();		/* source direct */
				current_byte = NEXT_BYTE();		/* destination direct */
				InternalRAM[current_index] = read_sfr(current_index);
				InternalRAM[current_byte] = InternalRAM[current_index];
				write_sfr_byte(current_byte);
				break;
			case MOV_dr_atR0:
				current_index = NEXT_BYTE();
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[current_index] = InternalRAM[InternalRAM[8*current_Rbank]];
				else
					InternalRAM[current_index] = InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max];
				write_sfr_byte(current_index);
				break;
			case MOV_dr_atR1:
				current_index = NEXT_BYTE();
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[current_index] = InternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
				else
					InternalRAM[current_index] = InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max];
				write_sfr_byte(current_index);
				break;
			case MOV_dr_R0:
			case MOV_dr_R1:
			case MOV_dr_R2:
			case MOV_dr_R3:
			case MOV_dr_R4:
			case MOV_dr_R5:
			case MOV_dr_R6:
			case MOV_dr_R7:
				current_index = NEXT_BYTE();
				InternalRAM[current_index] = InternalRAM[(current_opcode - 0x88) + 8*current_Rbank];
				write_sfr_byte(current_index);
				break;
			case MOV_DPTR:
				InternalRAM[DPH] = NEXT_BYTE();
				InternalRAM[DPL] = NEXT_BYTE();
				break;
			case MOV_bit_C:
				bit_address = NEXT_BYTE();
				bit_change(bit_address, cy);
				write_sfr_byte(bit_addressable(bit_address));
				break;
			case MOVC_DPTR:
				dptr = comb(InternalRAM[DPH],InternalRAM[DPL]);
				InternalRAM[ACC] = Program8051[dptr + InternalRAM[ACC]];
				break;
			case SUBB_im:
				current_byte = NEXT_BYTE();
				x = InternalRAM[ACC];
				InternalRAM[ACC] -= current_byte - cy;
				PSW_update_subb(x,InternalRAM[ACC]);
				break;
			case SUBB_dr:
				current_index = NEXT_BYTE();
				InternalRAM[current_index] = read_sfr(current_index);
				x = InternalRAM[ACC];
				InternalRAM[ACC] -= InternalRAM[current_index] - cy;
				PSW_update_subb(x,InternalRAM[ACC]);
				break;
			case SUBB_atR0:
				x = InternalRAM[ACC];
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] -= InternalRAM[InternalRAM[8*current_Rbank]] - cy;
				else
					InternalRAM[ACC] -= InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] - cy;
				PSW_update_subb(x,InternalRAM[ACC]);
				break;
			case SUBB_atR1:
				x = InternalRAM[ACC];
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] -= InternalRAM[InternalRAM[0x01 + 8*current_Rbank]] - cy;
				else
					InternalRAM[ACC] -= InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] - cy;
				PSW_update_subb(x,InternalRAM[ACC]);
				break;
			case SUBB_R0:
			case SUBB_R1:
			case SUBB_R2:
			case SUBB_R3:
			case SUBB_R4:
			case SUBB_R5:
			case SUBB_R6:
			case SUBB_R7:
				x = InternalRAM[ACC];
				InternalRAM[ACC] -= InternalRAM[(current_opcode - 0x98) + 8*current_Rbank] - cy;
				PSW_update_subb(x,InternalRAM[ACC]);
				break;
			case ORL_C_nbit:
				bit_address = NEXT_BYTE();
				bit_value = (~bit_check(bit_address) & 0b00000001);
				InternalRAM[PSW] = (InternalRAM[PSW] & 0b01111111) | ((bit_value | cy) << 7);
				break;
			case MOV_C_bit:
				bit_address = NEXT_BYTE();
				bit_value = bit_check(bit_address);
				InternalRAM[PSW] = (InternalRAM[PSW] & 0b01111111) | ((bit_value) << 7);
				break;
			case INC_DPTR:
				dptr = comb(InternalRAM[DPH],InternalRAM[DPL]);
				++dptr;
				InternalRAM[DPH] = decomp_high(dptr);
				InternalRAM[DPL] = decomp_low(dptr);
				break;
			case MUL:
				z = InternalRAM[ACC] * InternalRAM[B];
				if (z > 0xFF)
					InternalRAM[PSW] = (InternalRAM[PSW] | 0b00000100) & 0b01111111;
				else
					InternalRAM[PSW] = InternalRAM[PSW] & 0b01111011;
				InternalRAM[ACC] = decomp_low(z);
				InternalRAM[B] = decomp_high(z);
				break;
			case RESERVE:	// is there an end case?
				break;
			case MOV_atR0_dr:		// check that there are not more instructions for SBUF_RX
				current_index = NEXT_BYTE();
				if (current_index == SBUF_TX)
					current_index = SBUF_RX;
				InternalRAM[current_index] = read_sfr(current_index);
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[InternalRAM[8*current_Rbank]] = InternalRAM[current_index];
				else 
					InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] = InternalRAM[current_index];
				break;
			case MOV_atR1_dr:
				current_index = NEXT_BYTE();
				if (current_index == SBUF_TX)
					current_index = SBUF_RX;
				InternalRAM[current_index] = read_sfr(current_index);
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[InternalRAM[0x01 + 8*current_Rbank]] = InternalRAM[current_index];
				else
					InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] = InternalRAM[current_index];
				break;
			case MOV_R0_dr:
			case MOV_R1_dr:
			case MOV_R2_dr:
			case MOV_R3_dr:
			case MOV_R4_dr:
			case MOV_R5_dr:
			case MOV_R6_dr:
			case MOV_R7_dr:
				current_index = NEXT_BYTE();
				if (current_index == SBUF_TX)
					current_index = SBUF_RX;
				InternalRAM[current_index] = read_sfr(current_index);
				InternalRAM[(current_opcode - 0x88) + 8*current_Rbank] = InternalRAM[current_index];
				break;
			case ANL_C_nbit:
				bit_address = NEXT_BYTE();
				bit_value = (~bit_check(bit_address) & 0b00000001);
				InternalRAM[PSW] = (InternalRAM[PSW] & 0b01111111) | ((bit_value & cy) << 7);
				break;
			case CPL_bit:
				bit_address = NEXT_BYTE();
				bit_toggle(bit_address);
				write_sfr_byte(bit_addressable(bit_address));
				break;
			case CPL_C:
				InternalRAM[PSW] = (~cy << 7) | (InternalRAM[PSW] & 0b01111111);
				break;
			case CJNE_im:
				current_byte = NEXT_BYTE();
				offset = NEXT_BYTE();
				if (InternalRAM[ACC] != current_byte) {
					pc += offset; //changed
				}
				break;
			case CJNE_dr: // stopped here
				current_index = NEXT_BYTE();
				offset = NEXT_BYTE();
				InternalRAM[current_index] = read_sfr(current_index);
				if (InternalRAM[ACC] != InternalRAM[current_index]) {
					pc += offset;	//changed
				}
				break;
			case CJNE_atR0:
				current_byte = NEXT_BYTE();
				offset = NEXT_BYTE();
				if (InternalRAM[8*current_Rbank] < Indirect_max) {
					if (InternalRAM[InternalRAM[8*current_Rbank]] != current_byte) {
						pc += offset;
					}
				}
				else {
					if (InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] != current_byte) {
						pc += offset;
					}
				}
				break;
			case CJNE_atR1:
				current_byte = NEXT_BYTE();
				offset = NEXT_BYTE();
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max) {
					if (InternalRAM[InternalRAM[8*current_Rbank]] != current_byte) {
						pc += offset;
					}
				}
				else {
					if (InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] != current_byte) {
						pc += offset;
					}
				}
				break;
			case CJNE_R0_im:
			case CJNE_R1_im:
			case CJNE_R2_im:
			case CJNE_R3_im:
			case CJNE_R4_im:
			case CJNE_R5_im:
			case CJNE_R6_im:
			case CJNE_R7_im:
				current_byte = NEXT_BYTE();
				offset = NEXT_BYTE();
				if (InternalRAM[(current_opcode - 0xB8) + 8*current_Rbank] != current_byte) {
					pc += offset;
				}
				break;
			case PUSH:
				current_index = NEXT_BYTE();
				InternalRAM[current_index] = read_sfr(current_index);
				push(InternalRAM[current_index]);
				break;
			case CLR_bit:
				bit_address = NEXT_BYTE();
				x = bit_address;
				bit_change(bit_address, 0x00);
				write_sfr_bit(bit_address,0x00);
				break;
			case CLR_C:
				InternalRAM[PSW] = InternalRAM[PSW] & 0b01111111;
				break;
			case SWAP:
				InternalRAM[ACC] = (InternalRAM[ACC] << 4) | (InternalRAM[ACC] >> 4);
				break;
			case XCH_dr:
				current_index = NEXT_BYTE();
				InternalRAM[current_index] = read_sfr(current_index);
				x = InternalRAM[ACC];
				InternalRAM[ACC] = InternalRAM[current_index];
				InternalRAM[current_index] = x;
				write_sfr_byte(current_index);
				break;
			case XCH_atR0:
				x = InternalRAM[ACC];
				if (InternalRAM[8*current_Rbank] < Indirect_max) {
					InternalRAM[ACC] = InternalRAM[InternalRAM[8*current_Rbank]];
					InternalRAM[InternalRAM[8*current_Rbank]] = x;
				}
				else {
					InternalRAM[ACC] = InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max];
					InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] = x;
				}
				break;
			case XCH_atR1:
				x = InternalRAM[ACC];
				if (InternalRAM[8*current_Rbank] < Indirect_max) {
					InternalRAM[ACC] = InternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
					InternalRAM[InternalRAM[0x01 + 8*current_Rbank]] = x;
				}
				else {
					InternalRAM[ACC] = InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max];
					InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] = x;
				}
				break;
			case XCH_R0:
			case XCH_R1:
			case XCH_R2:
			case XCH_R3:
			case XCH_R4:
			case XCH_R5:
			case XCH_R6:
			case XCH_R7:
				x = InternalRAM[ACC];
				InternalRAM[ACC] = InternalRAM[(current_opcode - 0xC8) + 8*current_Rbank];
				InternalRAM[(current_opcode - 0xC8) + 8*current_Rbank] = x;
				break;
			case POP:
				current_index = NEXT_BYTE();
				InternalRAM[current_index] = pop();
				break;
			case SETB_bit:
				bit_address = NEXT_BYTE();
				bit_change(bit_address, 0x01);
				write_sfr_bit(bit_address, 0x01);
				break;
			case SETB_C:
				InternalRAM[PSW] = InternalRAM[PSW] | 0b10000000;
				break;
			case DA:		/* Decimal adjust (binary-coded decimal) */
				if (((InternalRAM[ACC] & 0b00001111) > 9) | (bit_check(AC) == 1)) {
					InternalRAM[ACC] += 0x06;
					x = 1;
				}
				else
					x = 0;
				if ((cy == 1) || ((x == 1) && (((InternalRAM[ACC] & 0b11110000) >> 4) > 9)))
					InternalRAM[ACC] += 0x60;
				break;
			case DJNZ_dr:
				current_index = NEXT_BYTE();
				offset = NEXT_BYTE();
				--InternalRAM[current_index];
				if (InternalRAM[current_index] != 0) {
					pc += offset;
				}
				write_sfr_byte(current_index);
				break;
			case XCHD_atR0:	
				if (InternalRAM[8*current_Rbank] < Indirect_max) {
					InternalRAM[ACC] = (InternalRAM[ACC] & 0b11110000) | (InternalRAM[InternalRAM[8*current_Rbank]] & 0b00001111);	
					InternalRAM[InternalRAM[8*current_Rbank]] = (InternalRAM[ACC] & 0b00001111) | (InternalRAM[InternalRAM[8*current_Rbank]] & 0b11110000);
				}
				else {
					InternalRAM[ACC] = (InternalRAM[ACC] & 0b11110000) | (InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] & 0b00001111);	
					InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] = (InternalRAM[ACC] & 0b00001111) | (InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] & 0b11110000);
				}
				break;
			case XCHD_atR1:
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max) {
					InternalRAM[ACC] = (InternalRAM[ACC] & 0b11110000) | (InternalRAM[InternalRAM[0x01 + 8*current_Rbank]] & 0b00001111);	
					InternalRAM[InternalRAM[0x01 + 8*current_Rbank]] = (InternalRAM[ACC] & 0b00001111) | (InternalRAM[InternalRAM[0x01 + 8*current_Rbank]] & 0b11110000);
				}
				else {
					InternalRAM[ACC] = (InternalRAM[ACC] & 0b11110000) | (InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] & 0b00001111);	
					InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] = (InternalRAM[ACC] & 0b00001111) | (InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] & 0b11110000);
				}
				break;
			case DJNZ_R0:
			case DJNZ_R1:
			case DJNZ_R2:
			case DJNZ_R3:
			case DJNZ_R4:
			case DJNZ_R5:
			case DJNZ_R6:
			case DJNZ_R7:
				offset = NEXT_BYTE();
				--InternalRAM[(current_opcode - 0xD8) + 8*current_Rbank];
				if (InternalRAM[(current_opcode - 0xD8) + 8*current_Rbank] != 0) {
					pc += offset;
				}
				break;
			case MOVX_A_DPTR:		// READ
				if (EA_flag)
					InternalRAM[ACC] = ExternalRAM[comb(InternalRAM[DPH],InternalRAM[DPL])];
				else {
					InternalRAM[ACC] = next_ex_RAM_read(InternalRAM[DPH],InternalRAM[DPL]);
					Xaccess_Write(0x00);
					}
				break;
			case MOVX_A_atR0:
				if (EA_flag)
					InternalRAM[ACC] = ExternalRAM[InternalRAM[8*current_Rbank]];
				else	{
					InternalRAM[ACC] = next_ex_RAM_read(0x00,InternalRAM[8*current_Rbank]);
					Xaccess_Write(0x00);
					}
				break;
			case MOVX_A_atR1:
				if (EA_flag)
					InternalRAM[ACC] = ExternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
				else	{
					InternalRAM[ACC] = next_ex_RAM_read(0x00,InternalRAM[0x01 + 8*current_Rbank]);
					Xaccess_Write(0x00);
					}
				break;
			case CLR_A:
				InternalRAM[ACC] = 0x00;
				break;
			case MOV_A_dr:
				current_index = NEXT_BYTE();
				if (current_index == SBUF_TX)
					current_index = SBUF_RX;
				InternalRAM[current_index] = read_sfr(current_index);
				InternalRAM[ACC] = InternalRAM[current_index];
				break;
			case MOV_A_atR0:
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] = InternalRAM[InternalRAM[8*current_Rbank]];
				else
					InternalRAM[ACC] = InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max];
				break;
			case MOV_A_atR1:
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[ACC] = InternalRAM[InternalRAM[0x01 + 8*current_Rbank]];
				else
					InternalRAM[ACC] = InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max];
				break;
			case MOV_A_R0:
			case MOV_A_R1:
			case MOV_A_R2:
			case MOV_A_R3:
			case MOV_A_R4:
			case MOV_A_R5:
			case MOV_A_R6:
			case MOV_A_R7:
				InternalRAM[ACC] = InternalRAM[(current_opcode - 0xE8) + 8*current_Rbank];
				break;
			case MOVX_DPTR_A:		// WRITE
				if (EA_flag)
					ExternalRAM[comb(InternalRAM[DPH],InternalRAM[DPL])] = InternalRAM[ACC];
				else {
					next_ex_RAM_write(InternalRAM[DPH],InternalRAM[DPL]);
					Xaccess_Write(0x00);
					}
				break;
			case MOVX_atR0_A:
				if (EA_flag)
					ExternalRAM[InternalRAM[8*current_Rbank]] = InternalRAM[ACC];
				else {
					next_ex_RAM_write(0x00,InternalRAM[8*current_Rbank]);
					Xaccess_Write(0x00);
					}
				break;
			case MOVX_atR1_A:
				if (EA_flag)
					ExternalRAM[InternalRAM[0x01 + 8*current_Rbank]] = InternalRAM[ACC];
				else {
					next_ex_RAM_write(0x00,InternalRAM[0x01 + 8*current_Rbank]);
					Xaccess_Write(0x00);
					}
				break;
			case CPL_A:
				InternalRAM[ACC] = ~InternalRAM[ACC];
				break;
			case MOV_dr_A:
				current_index = NEXT_BYTE();
				InternalRAM[current_index] = InternalRAM[ACC];
				write_sfr_byte(current_index);
				break;
			case MOV_atR0_A:
				if (InternalRAM[8*current_Rbank] < Indirect_max)
					InternalRAM[InternalRAM[8*current_Rbank]] = InternalRAM[ACC];
				else
					InternalRAM_indirect[InternalRAM[8*current_Rbank] - Indirect_max] = InternalRAM[ACC];
				break;
			case MOV_atR1_A:
				if (InternalRAM[0x01 + 8*current_Rbank] < Indirect_max)
					InternalRAM[InternalRAM[0x01 + 8*current_Rbank]] = InternalRAM[ACC];
				else
					InternalRAM_indirect[InternalRAM[0x01 + 8*current_Rbank] - Indirect_max] = InternalRAM[ACC];
				break;
			case MOV_R0_A:
			case MOV_R1_A:
			case MOV_R2_A:
			case MOV_R3_A:
			case MOV_R4_A:
			case MOV_R5_A:
			case MOV_R6_A:
			case MOV_R7_A:
				InternalRAM[(current_opcode - 0xF8) + 8*current_Rbank] = InternalRAM[ACC];
				break;
		}
		globalPc = pc;
		CyGlobalIntEnable;	
     }
	}
	
#endif

/* [] END OF FILE */
